@@ -230,20 +230,15 @@ class LinuxSCSITestCase(base.TestCase):
               {'do_raise': True, 'force': True})
     @ddt.unpack
     @mock.patch.object(linuxscsi.LinuxSCSI, '_remove_scsi_symlinks')
-    @mock.patch.object(linuxscsi.LinuxSCSI, 'multipath_del_path')
-    @mock.patch.object(linuxscsi.LinuxSCSI, 'is_multipath_running',
-                       return_value=True)
     @mock.patch.object(linuxscsi.LinuxSCSI, 'flush_multipath_device')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'get_dm_name')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'find_sysfs_multipath_dm')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'wait_for_volumes_removal')
-    @mock.patch.object(linuxscsi.LinuxSCSI, 'remove_scsi_device')
+    @mock.patch('os_brick.initiator.linuxscsi.LinuxSCSI.remove_scsi_device')
     def test_remove_connection_multipath_complete(self, remove_mock, wait_mock,
                                                   find_dm_mock,
                                                   get_dm_name_mock,
                                                   flush_mp_mock,
-                                                  is_mp_running_mock,
-                                                  mp_del_path_mock,
                                                   remove_link_mock,
                                                   do_raise, force):
         if do_raise:
@@ -251,6 +246,7 @@ class LinuxSCSITestCase(base.TestCase):
         devices_names = ('sda', 'sdb')
         exc = exception.ExceptionChainer()
         mp_name = self.linuxscsi.remove_connection(devices_names,
+                                                   is_multipath=True,
                                                    force=mock.sentinel.Force,
                                                    exc=exc)
         find_dm_mock.assert_called_once_with(devices_names)
@@ -258,9 +254,6 @@ class LinuxSCSITestCase(base.TestCase):
         flush_mp_mock.assert_called_once_with(get_dm_name_mock.return_value)
         self.assertEqual(get_dm_name_mock.return_value if do_raise else None,
                          mp_name)
-        is_mp_running_mock.assert_not_called()
-        mp_del_path_mock.assert_has_calls([
-            mock.call('/dev/sda'), mock.call('/dev/sdb')])
         remove_mock.assert_has_calls([
             mock.call('/dev/sda', mock.sentinel.Force, exc, False),
             mock.call('/dev/sdb', mock.sentinel.Force, exc, False)])
@@ -269,46 +262,6 @@ class LinuxSCSITestCase(base.TestCase):
         remove_link_mock.assert_called_once_with(devices_names)
 
     @mock.patch.object(linuxscsi.LinuxSCSI, '_remove_scsi_symlinks')
-    @mock.patch.object(linuxscsi.LinuxSCSI, 'multipath_del_path')
-    @mock.patch.object(linuxscsi.LinuxSCSI, 'is_multipath_running',
-                       return_value=True)
-    @mock.patch.object(linuxscsi.LinuxSCSI, 'flush_multipath_device')
-    @mock.patch.object(linuxscsi.LinuxSCSI, 'get_dm_name')
-    @mock.patch.object(linuxscsi.LinuxSCSI, 'find_sysfs_multipath_dm',
-                       return_value=None)
-    @mock.patch.object(linuxscsi.LinuxSCSI, 'wait_for_volumes_removal')
-    @mock.patch.object(linuxscsi.LinuxSCSI, 'remove_scsi_device')
-    def test_remove_connection_multipath_complete_no_dm(self, remove_mock,
-                                                        wait_mock,
-                                                        find_dm_mock,
-                                                        get_dm_name_mock,
-                                                        flush_mp_mock,
-                                                        is_mp_running_mock,
-                                                        mp_del_path_mock,
-                                                        remove_link_mock):
-        devices_names = ('sda', 'sdb')
-        exc = exception.ExceptionChainer()
-        mp_name = self.linuxscsi.remove_connection(devices_names,
-                                                   force=mock.sentinel.Force,
-                                                   exc=exc)
-        find_dm_mock.assert_called_once_with(devices_names)
-        get_dm_name_mock.assert_not_called()
-        flush_mp_mock.assert_not_called()
-        self.assertIsNone(mp_name)
-        is_mp_running_mock.assert_called_once()
-        mp_del_path_mock.assert_has_calls([
-            mock.call('/dev/sda'), mock.call('/dev/sdb')])
-        remove_mock.assert_has_calls([
-            mock.call('/dev/sda', mock.sentinel.Force, exc, False),
-            mock.call('/dev/sdb', mock.sentinel.Force, exc, False)])
-        wait_mock.assert_called_once_with(devices_names)
-        self.assertFalse(bool(exc))
-        remove_link_mock.assert_called_once_with(devices_names)
-
-    @mock.patch.object(linuxscsi.LinuxSCSI, '_remove_scsi_symlinks')
-    @mock.patch.object(linuxscsi.LinuxSCSI, 'multipath_del_path')
-    @mock.patch.object(linuxscsi.LinuxSCSI, 'is_multipath_running',
-                       return_value=True)
     @mock.patch.object(linuxscsi.LinuxSCSI, 'flush_multipath_device',
                        side_effect=Exception)
     @mock.patch.object(linuxscsi.LinuxSCSI, 'get_dm_name')
@@ -317,51 +270,32 @@ class LinuxSCSITestCase(base.TestCase):
     @mock.patch.object(linuxscsi.LinuxSCSI, 'remove_scsi_device')
     def test_remove_connection_multipath_fail(self, remove_mock, wait_mock,
                                               find_dm_mock, get_dm_name_mock,
-                                              flush_mp_mock,
-                                              is_mp_running_mock,
-                                              mp_del_path_mock,
-                                              remove_link_mock):
+                                              flush_mp_mock, remove_link_mock):
         flush_mp_mock.side_effect = exception.ExceptionChainer
         devices_names = ('sda', 'sdb')
         exc = exception.ExceptionChainer()
         self.assertRaises(exception.ExceptionChainer,
                           self.linuxscsi.remove_connection,
-                          devices_names, force=False, exc=exc)
+                          devices_names, is_multipath=True,
+                          force=False, exc=exc)
         find_dm_mock.assert_called_once_with(devices_names)
         get_dm_name_mock.assert_called_once_with(find_dm_mock.return_value)
         flush_mp_mock.assert_called_once_with(get_dm_name_mock.return_value)
-        is_mp_running_mock.assert_not_called()
-        mp_del_path_mock.assert_not_called()
         remove_mock.assert_not_called()
         wait_mock.assert_not_called()
         remove_link_mock.assert_not_called()
         self.assertTrue(bool(exc))
 
     @mock.patch.object(linuxscsi.LinuxSCSI, '_remove_scsi_symlinks')
-    @mock.patch.object(linuxscsi.LinuxSCSI, 'multipath_del_path')
-    @mock.patch.object(linuxscsi.LinuxSCSI, 'is_multipath_running',
-                       return_value=True)
-    @mock.patch.object(linuxscsi.LinuxSCSI, 'find_sysfs_multipath_dm')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'wait_for_volumes_removal')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'remove_scsi_device')
-    def test_remove_connection_singlepath_no_path(self, remove_mock, wait_mock,
-                                                  find_dm_mock,
-                                                  is_mp_running_mock,
-                                                  mp_del_path_mock,
-                                                  remove_link_mock):
-        # Test remove connection when we didn't form a multipath and didn't
-        # even use any of the devices that were found.  This means that we
-        # don't flush any of the single paths when removing them.
-        find_dm_mock.return_value = None
+    def test_remove_connection_singlepath(self, remove_mock, wait_mock,
+                                          remove_link_mock):
         devices_names = ('sda', 'sdb')
         exc = exception.ExceptionChainer()
-        self.linuxscsi.remove_connection(devices_names,
+        self.linuxscsi.remove_connection(devices_names, is_multipath=False,
                                          force=mock.sentinel.Force,
                                          exc=exc)
-        find_dm_mock.assert_called_once_with(devices_names)
-        is_mp_running_mock.assert_called_once()
-        mp_del_path_mock.assert_has_calls([
-            mock.call('/dev/sda'), mock.call('/dev/sdb')])
         remove_mock.assert_has_calls(
             [mock.call('/dev/sda', mock.sentinel.Force, exc, False),
              mock.call('/dev/sdb', mock.sentinel.Force, exc, False)])
@@ -369,33 +303,19 @@ class LinuxSCSITestCase(base.TestCase):
         remove_link_mock.assert_called_once_with(devices_names)
 
     @mock.patch.object(linuxscsi.LinuxSCSI, '_remove_scsi_symlinks')
-    @mock.patch.object(linuxscsi.LinuxSCSI, 'multipath_del_path')
-    @mock.patch.object(linuxscsi.LinuxSCSI, 'is_multipath_running',
-                       return_value=False)
-    @mock.patch.object(linuxscsi.LinuxSCSI, 'find_sysfs_multipath_dm')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'wait_for_volumes_removal')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'remove_scsi_device')
     def test_remove_connection_singlepath_used(self, remove_mock, wait_mock,
-                                               find_dm_mock,
-                                               is_mp_running_mock,
-                                               mp_del_path_mock,
                                                remove_link_mock):
-        # Test remove connection when we didn't form a multipath and just used
-        # one of the single paths that were found.  This means that we don't
-        # flush any of the single paths when removing them.
-        find_dm_mock.return_value = None
         devices_names = ('sda', 'sdb')
         exc = exception.ExceptionChainer()
 
         # realpath was mocked on test setup
         with mock.patch('os.path.realpath', side_effect=self.realpath):
-            self.linuxscsi.remove_connection(devices_names,
+            self.linuxscsi.remove_connection(devices_names, is_multipath=True,
                                              force=mock.sentinel.Force,
                                              exc=exc, path_used='/dev/sdb',
                                              was_multipath=False)
-        find_dm_mock.assert_called_once_with(devices_names)
-        is_mp_running_mock.assert_called_once()
-        mp_del_path_mock.assert_not_called()
         remove_mock.assert_has_calls(
             [mock.call('/dev/sda', mock.sentinel.Force, exc, False),
              mock.call('/dev/sdb', mock.sentinel.Force, exc, True)])
@@ -928,6 +848,21 @@ loop0                                     0"""
         mock_exec.assert_called_once_with(
             'multipathd', 'show', 'status', run_as_root=True, root_helper=None)
 
+    def test_get_device_info(self):
+        ret = "/dev/sg0 scsi1 channel=1 id=0 lun=0 [em]\n"
+        with mock.patch.object(self.linuxscsi, '_execute') as exec_mock:
+            exec_mock.return_value = (ret, "")
+            info = self.linuxscsi.get_device_info('/dev/adevice')
+
+            exec_mock.assert_called_once_with('sg_scan', '/dev/adevice',
+                                              root_helper=None,
+                                              run_as_root=True)
+            self.assertEqual(info, {'channel': '1',
+                                    'device': '/dev/adevice',
+                                    'host': '1',
+                                    'id': '0',
+                                    'lun': '0'})
+
     @mock.patch('six.moves.builtins.open')
     def test_get_sysfs_wwn_mpath(self, open_mock):
         wwn = '3600d0230000000000e13955cc3757800'
@@ -1176,10 +1111,6 @@ loop0                                     0"""
         self.linuxscsi.multipath_add_path('/dev/sda')
         self.assertEqual(['multipathd add path /dev/sda'], self.cmds)
 
-    def test_multipath_del_path(self):
-        self.linuxscsi.multipath_del_path('/dev/sda')
-        self.assertEqual(['multipathd del path /dev/sda'], self.cmds)
-
     @ddt.data({'con_props': {}, 'dev_info': {'path': mock.sentinel.path}},
               {'con_props': None, 'dev_info': {'path': mock.sentinel.path}},
               {'con_props': {'device_path': mock.sentinel.device_path},
@@ -1210,15 +1141,8 @@ loop0                                     0"""
     def test_get_dev_path_no_path(self, con_props, dev_info):
         self.assertEqual('', self.linuxscsi.get_dev_path(con_props, dev_info))
 
-    @ddt.data(('/dev/sda', '/dev/sda', False, True, None),
-              # This checks that we ignore the was_multipath parameter if it
-              # doesn't make sense (because the used path is the one we are
-              # asking about)
-              ('/dev/sda', '/dev/sda', True, True, None),
+    @ddt.data(('/dev/sda', '/dev/sda', True, False, None),
               ('/dev/sda', '', True, False, None),
-              # Check for encrypted volume
-              ('/dev/link_sda', '/dev/disk/by-path/pci-XYZ', False, True,
-               ('/dev/sda', '/dev/mapper/crypt-pci-XYZ')),
               ('/dev/link_sda', '/dev/link_sdb', False, False, ('/dev/sda',
                                                                 '/dev/sdb')),
               ('/dev/link_sda', '/dev/link2_sda', False, True, ('/dev/sda',
